/*
 * IntervalTimer.h
 *
 *  Created on: Aug 28, 2015
 *      Author: eric
 */

#ifndef INCLUDE_INTERVALTIMER_H_
#define INCLUDE_INTERVALTIMER_H_

#include <fastec/fi_exception.h>
#include <fastec/statistics.h>
#include <fastec/port.h>
#include <time.h>

namespace Fastec
{

/**
 * @brief Measure and accumulate statistics of time intervals
 *
 * Generally used to collect execution time data for various processes
 * in the LRS. This can be used in two ways:
 * - Measure intervals from a 'start' time to a 'stop' time.
 * - Measure periods between successive 'snap' times.
 */
class IntervalTimer : public Statistics
{
public:
    IntervalTimer();
    virtual ~IntervalTimer();

    void start();	/**< @brief Start a time interval */
    void stop();    /**< @brief Stop a time interval */
    void snap();	/**< @brief Mark a periodic interval */
    void reset();   /**< @brief Reset timer and associated statistics */
    void dump(FILE* fd);						/**< @brief Print (dump) statistics to a file */
    void dump(FILE* fd, const char* tagStr);	/**< @brief Print (dump) statistics to a file, with a label/tag */

    double lastMeasurement;

private:
    struct timespec ts_start; /**< Remember start time or last 'snap' time */
};

} /* namespace Fastec */

#endif /* INCLUDE_INTERVALTIMER_H_ */
