#ifndef INCLUDE_FASTEC_INTERNAL_SYS_STATISTICS_H
#define INCLUDE_FASTEC_INTERNAL_SYS_STATISTICS_H

#include <mutex>

#include <stdio.h>
#include <float.h>

namespace Fastec
{

class Statistics
{
public:
    Statistics();
    virtual ~Statistics();

    void accumulate(double x);
    virtual void reset(void);

    void startHistogram(double l, double h, unsigned b);

    double getMean(void);
    double getMax(void);
    double getMin(void);
    long getNSamples(void);

    int dump(FILE* fd);							/**< @brief Print (dump) statistics to a file */
    int dump(FILE* fd, const char* tagStr);	    /**< @brief Print (dump) statistics to a file, with a label/tag */
    void dumpHist(FILE* fd);                    /**< @brief Print (dump) histogram */

private:
    std::mutex *mutex;

    double max;
    double min;

    /* Knuth, Volume 2 2nd. Edition, page 216 [Welford 1962] */
    long k;
    double Mk;
    double Sk;

    /* Histogram */
    double low;
    double high;
    unsigned nBins;
    long* hist;
};

} /* end namespace Fastec */

#endif /* INCLUDE_FASTEC_INTERNAL_SYS_STATISTICS_H */
